<?php
class benchmarkemailModelPps extends modelPps {
	private $_clients = array();
	private $_lastRequireConfirm = false;

	private $_CIPHER = null;//MCRYPT_RIJNDAEL_128; // Rijndael-128 is AES
	private $_MODE   = null;//MCRYPT_MODE_CBC;
	private $_username = '';
	private $_password = '';
	private $_maxStrLength = 255;
	private $_apiUrl = 'https://api.benchmarkemail.com/1.3/';
	public function getClient($login, $pass) {
		$clientKey = md5($login. '_'. $pass);
		if(isset($this->_clients[ $clientKey ]))
			return $this->_clients[ $clientKey ];
		if(!class_exists('IXR_Client')) {
			require_once( ABSPATH . WPINC . '/class-IXR.php' );
		}
		$this->_clients[ $clientKey ] = new IXR_Client( $this->_apiUrl, false, 443, 20 );
		if($this->_clients[ $clientKey ]->error) {
			$this->pushError( $this->_clients[ $clientKey ]->error->message );
			return false;
		}
		return $this->_clients[ $clientKey ];
	}
	public function callClientFunction() {
		$login = $this->getApiUsername();
		$password = $this->getApiPassword();
		$args = func_get_args();
		$client = $this->getClient($login, $password);
		call_user_func_array( array( $client, 'query' ), $args );
		$response =  $client->getResponse();
		if ($client->isError()){
			$this->pushError( $client->getErrorMessage() );
		}
		return $response;
	}
	public function getLists($d) {
		$login = $this->getApiUsername();
		$password = $this->getApiPassword();
		if(!empty($login)) {
			if(!empty($password)) {
				$client = $this->getClient($login, $password);
				if($client) {
					$token = $this->callClientFunction('login', $login, $password);
					if(!$this->haveErrors()) {
						$listsRes = $this->callClientFunction('listGet', $token, "", 1, 100, "", "");
						if(!$this->haveErrors()) {
							if(!empty($listsRes) && count($listsRes) > 0) {
								$listsData = array();
								foreach($listsRes as $l) {
									$listsData[] = array('id' => $l['id'], 'name' => $l['listname']);
								}
								return $listsData;
							} else {
								$this->pushError(__('Please make sure that you created some Lists under your Benchmark account.', PPS_LANG_CODE));
							}
						}
					}
				}
			} else
				$this->pushError(__('Please enter your Benchmark Password', PPS_LANG_CODE), 'sub_be_pass');
		} else
			$this->pushError(__('Please enter your Benchmark Login', PPS_LANG_CODE), 'sub_be_login');
		return false;

	}
	public function subscribe($d, $popup, $validateIp = false) {
		$email = isset($d['email']) ? trim($d['email']) : false;
		if(!empty($email)) {
			if(is_email($email)) {
				$lists = isset($popup['params']['tpl']['sub_be_lists']) ? $popup['params']['tpl']['sub_be_lists'] : array();
				if(!empty($lists)) {
					if(!$validateIp || $validateIp && framePps::_()->getModule('subscribe')->getModel()->checkOftenAccess(array('only_check' => true))) {
						$login = $this->getApiUsername();
						$password = $this->getApiPassword();
						$client =  $this->getClient($login, $password);
						if($client) {
							$addData = array('email' => $email);
							$name = isset($d['name']) ? trim($d['name']) : '';
							if(!empty($name)) {
								$firstLastName = explode(' ', $name);
								$addData['firstname'] = $firstLastName[ 0 ];
								if(isset($firstLastName[ 1 ]) && !empty($firstLastName[ 1 ])) {
									$addData['lastname'] = $firstLastName[ 1 ];
								}
							}
							if(isset($popup['params']['tpl']['sub_fields'])
								&& !empty($popup['params']['tpl']['sub_fields'])
							) {
								foreach($popup['params']['tpl']['sub_fields'] as $k => $f) {
									if(in_array($k, array('name', 'email'))) continue;	// Ignore standard fields
									if(isset($d[ $k ])) {
										$addData[ $k ] = $d[ $k ];
									}
								}
							}
							foreach($lists as $listId) {
								$token = $this->callClientFunction('login', $login, $password);
								if(!$this->haveErrors()) {
									$this->callClientFunction('listAddContacts', $token, $listId, array($addData));
									if($this->haveErrors()) {
										return false;
									}
								}
							}
							return true;
						}
					}
				} else
					$this->pushError (__('No lists to add selected in admin area - contact site owner to resolve this issue.', PPS_LANG_CODE));
			} else
				$this->pushError (framePps::_()->getModule('subscribe')->getModel()->getInvalidEmailMsg($popup), 'email');
		} else
			$this->pushError (framePps::_()->getModule('subscribe')->getModel()->getInvalidEmailMsg($popup), 'email');
		return false;
	}
	public function requireConfirm() {
		if($this->_lastRequireConfirm)
			return true;
		$destData = framePps::_()->getModule('subscribe')->getDestByKey( $this->getCode() );
		return $destData['require_confirm'];
	}
	private function _encrypt($pureString, $encryptionKey = '') {
		$encryptedString = base64_encode($pureString);
		return $encryptedString. '_enc';
	}
	private function _decrypt($encryptedString, $encryptionKey = '') {
		if(function_exists('mcrypt_decrypt') && substr($encryptedString, -4) !== '_enc') {
			if(empty($encryptionKey))
				$encryptionKey = $this->_getEncriptKey();
			$this->_initCript();
			$encryptionKey = substr($encryptionKey, 0, 16);
			$ciphertext = base64_decode($encryptedString);
			$ivSize = @mcrypt_get_iv_size($this->_CIPHER, $this->_MODE);
			if (strlen($ciphertext) < $ivSize) {
				return false;
			}
			$iv = substr($ciphertext, 0, $ivSize);
			$ciphertext = substr($ciphertext, $ivSize);
			$plaintext = @mcrypt_decrypt($this->_CIPHER, $encryptionKey, $ciphertext, $this->_MODE, $iv);
			return rtrim($plaintext, "\0");
		} else {
			if(substr($encryptedString, -4) === '_enc') {
				$encryptedString = substr($encryptedString, 0, -4);
			}
			return base64_decode($encryptedString);
		}
	}
	private function _initCript() {
		if(defined('MCRYPT_RIJNDAEL_128')) {
			$this->_CIPHER = MCRYPT_RIJNDAEL_128; // Rijndael-128 is AES
		}
		if(defined('MCRYPT_MODE_CBC')) {
			$this->_MODE = MCRYPT_MODE_CBC;
		}
	}
	private function _getEncriptKey() {
		$authKey = AUTH_KEY;
		if(strlen($authKey) < 16) {
			for($i = strlen($authKey); $i < 16; $i++) {
				$authKey .= '1';
			}
		}
		return $authKey;
	}
	public function getApiUsername() {
		if(empty($this->_username)) {
			$this->_username = $this->_decrypt( framePps::_()->getModule('options')->get($this->getCode(). '_username') );
		}
		return $this->_username;
	}
	public function getApiPassword() {
		if(empty($this->_password)) {
			$this->_password = $this->_decrypt( framePps::_()->getModule('options')->get($this->getCode(). '_password') );
		}
		return $this->_password;
	}
	public function setApiUsername($username) {
		$username = strlen($username) > $this->_maxStrLength ? '' : trim($username);
		framePps::_()->getModule('options')->getModel()->save($this->getCode(). '_username', $this->_encrypt($username));
	}
	public function setApiPassword($password) {
		$password = strlen($password) > $this->_maxStrLength ? '' : trim($password);
		framePps::_()->getModule('options')->getModel()->save($this->getCode(). '_password', $this->_encrypt($password));
	}
}